@@ -24,6 +24,7 @@
 #include <sys/types.h>
 #include <errno.h>
 
+#include <glib.h>
 #include <glib/gi18n.h>
 
 #include <gdk/gdk.h>
@@ -49,14 +50,7 @@
 #include <libgames-support/games-atk-utils.h>
 #endif
 
-#ifdef HAVE_CLUTTER
-#include "ar-clutter-embed.h"
-#include "ar-style.h"
-#include "baize.h"
-#include "board.h"
-#else
 #include "board-noclutter.h"
-#endif
 
 #include "ar-card-theme.h"
 #include "ar-card-themes.h"
@@ -95,13 +89,6 @@
 #define C_(context, string) (_(string))
 #endif
 
-/* On maemo5, the toolbar doesn't have enough space to show a
- * significant amount of text, so we always use the banner here.
- */
-#if defined(HAVE_HILDON) && (defined(HAVE_MAEMO_3) || defined(HAVE_MAEMO_4))
-#define MAEMO_TOOLBAR_BANNER
-#endif
-
 /* On maemo5, there's no hardware key to exit the fullscreen mode. So we show
  * an overlay button to restore normal mode, if the toolbar is hidden too.
  */
@@ -119,11 +106,9 @@ enum
   ACTION_OPTIONS_MENU,
   ACTION_DEAL,
   ACTION_HINT,
+  ACTION_LEAVE_FULLSCREEN,
 #ifdef HAVE_HILDON
   ACTION_ACCEL_UNDO_MOVE,
-#endif
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
-  ACTION_LEAVE_FULLSCREEN,
 #endif
   LAST_ACTION
 };
@@ -132,22 +117,13 @@ struct _AisleriotWindowPrivate
 {
   AisleriotGame *game;
   ArStyle *board_style;
-#ifdef HAVE_CLUTTER
-  ArClutterEmbed *board;
-  ClutterActor *baize_actor;
-  ClutterActor *board_actor;
-#else
   AisleriotBoard *board;
-#endif
 
   ArCardThemes *theme_manager;
   ArCardTheme *theme;
 
 #ifdef HAVE_HILDON
   guint game_message_hash;
-#ifdef MAEMO_TOOLBAR_BANNER
-  GtkLabel *game_message_label;
-#endif /* MAEMO_TOOLBAR_BANNER */
 #else
   GtkStatusbar *statusbar;
   guint game_message_id;
@@ -383,11 +359,7 @@ undo_cb (GtkAction *action,
   AisleriotWindowPrivate *priv = window->priv;
 
   /* If a move is in progress, cancel it before changing the game! */
-#ifdef HAVE_CLUTTER
-  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
-#else
   aisleriot_board_abort_move (priv->board);
-#endif
 
   aisleriot_game_undo_move (priv->game);
 }
@@ -398,11 +370,7 @@ redo_cb (GtkAction *action,
 {
   AisleriotWindowPrivate *priv = window->priv;
 
-#ifdef HAVE_CLUTTER
-  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
-#else
   aisleriot_board_abort_move (priv->board);
-#endif
 
   aisleriot_game_redo_move (priv->game);
 }
@@ -571,7 +539,7 @@ statistics_cb (GtkAction *action,
   gtk_window_present (GTK_WINDOW (priv->stats_dialog));
 }
 
-#ifdef ENABLE_CARD_THEMES_INSTALLER
+#ifndef HAVE_HILDON
 
 static void
 install_themes_cb (GtkAction *action,
@@ -584,7 +552,7 @@ install_themes_cb (GtkAction *action,
                                     gtk_get_current_event_time ());
 }
 
-#endif /* ENABLE_CARD_THEMES_INSTALLER */
+#endif /* !HAVE_HILDON */
 
 #ifdef ENABLE_DEBUG_UI
 
@@ -842,7 +810,11 @@ debug_choose_seed_cb (GtkAction *action,
   g_snprintf (str, sizeof (str), "%u", aisleriot_game_get_seed (priv->game));
   entry = gtk_entry_new ();
   gtk_entry_set_text (GTK_ENTRY (entry), str);
+#if GTK_CHECK_VERSION (2, 90, 5)
+  gtk_box_pack_end (GTK_BOX (gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog))), entry, FALSE, FALSE, 0);
+#else
   gtk_box_pack_end (GTK_BOX (GTK_MESSAGE_DIALOG (dialog)->label->parent), entry, FALSE, FALSE, 0);
+#endif
   gtk_widget_show (entry);
   g_object_set_data (G_OBJECT (dialog), "entry", entry);
   gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
@@ -949,13 +921,11 @@ set_fullscreen_actions (AisleriotWindow *window,
   g_object_set (priv->main_menu, "visible", !is_fullscreen, NULL);
 #endif /* HAVE_MAEMO */
 
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
   gtk_action_set_visible (priv->action[ACTION_LEAVE_FULLSCREEN], is_fullscreen);
   g_object_set (gtk_ui_manager_get_widget (priv->ui_manager, "/Toolbar/LeaveFullscreenSep"),
                 "visible", is_fullscreen,
                 "draw", FALSE,
                 NULL);
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (priv->action[ACTION_FULLSCREEN]),
                                 is_fullscreen);
@@ -977,8 +947,6 @@ fullscreen_toggled_cb (GtkToggleAction *action,
   }
 }
 
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
-
 static void
 leave_fullscreen_cb (GtkAction *action,
                      GtkWindow *window)
@@ -986,8 +954,6 @@ leave_fullscreen_cb (GtkAction *action,
   gtk_window_unfullscreen (window);
 }
 
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
-
 static void
 clickmove_toggle_cb (GtkToggleAction *action,
                      AisleriotWindow *window)
@@ -1020,24 +986,6 @@ sound_toggle_cb (GtkToggleAction *action,
 
 #endif /* ENABLE_SOUND */
 
-#ifdef HAVE_CLUTTER
-
-static void
-animations_toggle_cb (GtkToggleAction *action,
-                      AisleriotWindow *window)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-  gboolean enabled;
-
-  enabled = gtk_toggle_action_get_active (action);
-
-  ar_style_set_enable_animations (priv->board_style, enabled);
-  
-  games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_ANIMATIONS), enabled);
-}
-
-#endif /* HAVE_CLUTTER */
-
 static void
 show_hint_cb (GtkAction *action,
               AisleriotWindow *window)
@@ -1756,9 +1704,6 @@ game_type_changed_cb (AisleriotGame *game,
   gtk_action_set_visible (priv->action[ACTION_DEAL], dealable);
 
 #ifdef HAVE_HILDON
-#ifdef MAEMO_TOOLBAR_BANNER
-  gtk_label_set_text (priv->game_message_label, NULL);
-#endif /* MAEMO_TOOLBAR_BANNER */
 #else
   games_clock_reset (GAMES_CLOCK (priv->clock));
 
@@ -1776,22 +1721,18 @@ static void
 game_new_cb (AisleriotGame *game,
              AisleriotWindow *window)
 {
-#if defined(MAEMO_TOOLBAR_BANNER) || !defined(HAVE_HILDON)
+#ifndef HAVE_HILDON
   AisleriotWindowPrivate *priv = window->priv;
 #endif
 
   update_statistics_display (window);
 
-#ifdef HAVE_HILDON
-#ifdef MAEMO_TOOLBAR_BANNER
-  gtk_label_set_text (priv->game_message_label, NULL);
-#endif /* MAEMO_TOOLBAR_BANNER */
-#else
+#ifndef HAVE_HILDON
   games_clock_reset (GAMES_CLOCK (priv->clock));
 
   gtk_statusbar_pop (priv->statusbar, priv->game_message_id);
   gtk_statusbar_pop (priv->statusbar, priv->board_message_id);
-#endif /* HAVE_HILDON */
+#endif /* !HAVE_HILDON */
 }
 
 static void
@@ -1801,16 +1742,8 @@ game_statusbar_message_cb (AisleriotGame *game,
 {
   AisleriotWindowPrivate *priv = window->priv;
 #ifdef HAVE_HILDON
-#ifdef MAEMO_TOOLBAR_BANNER
-  gtk_label_set_text (priv->game_message_label, message);
-#endif
 
-  /* The banners are annoying, but until I get a better idea, use them
-   * when the toolbar is hidden.
-   */
-#ifdef MAEMO_TOOLBAR_BANNER
-  if (!priv->toolbar_visible)
-#endif
+  /* The banners are annoying, but until I get a better idea, use them */
   {
     guint hash;
 
@@ -1939,7 +1872,7 @@ game_exception_cb (AisleriotGame *game,
   gtk_widget_show (dialog);
 }
 
-#if defined(HAVE_CLUTTER) || (defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0))
+#if defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0)
 
 static void
 settings_changed_cb (GtkSettings *settings,
@@ -1956,23 +1889,12 @@ settings_changed_cb (GtkSettings *settings,
   else
     name = NULL;
 
-#ifdef HAVE_CLUTTER
-  if (name == NULL || strcmp (name, "gtk-enable-animations") == 0) {
-    g_object_get (settings, "gtk-enable-animations", &enabled, NULL);
-
-    action = gtk_action_group_get_action (priv->action_group, "Animations");
-    gtk_action_set_visible (action, enabled);
-  }
-#endif /* HAVE_CLUTTER */
-
-#if defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0)
   if (name == NULL || strcmp (name, "gtk-enable-event-sounds") == 0) {
     g_object_get (settings, "gtk-enable-event-sounds", &enabled, NULL);
 
     action = gtk_action_group_get_action (priv->action_group, "Sound");
     gtk_action_set_visible (action, enabled);
   }
-#endif /* ENABLE_SOUND && GTK >= 2.14 */
 }
 
 static void
@@ -2000,17 +1922,11 @@ screen_changed_cb (GtkWidget *widget,
 
   settings = gtk_widget_get_settings (widget);
   settings_changed_cb (settings, NULL, window);
-#ifdef HAVE_CLUTTER
-  g_signal_connect (settings, "notify::gtk-enable-animations",
-                    G_CALLBACK (settings_changed_cb), window);
-#endif
-#if defined (ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0)
   g_signal_connect (settings, "notify::gtk-enable-event-sounds",
                     G_CALLBACK (settings_changed_cb), window);
-#endif
 }
 
-#endif /* HAVE_CLUTTER || ENABLE_SOUND && GTK+ >= 2.14.0 */
+#endif /* ENABLE_SOUND && GTK+ >= 2.14.0 */
 
 /*
  * aisleriot_window_set_freecell_mode:
@@ -2060,37 +1976,6 @@ board_status_message_cb (AisleriotBoard *board,
 
 #endif /* !HAVE_HILDON */
 
-#ifdef HAVE_CLUTTER
-
-static void
-board_cursor_cb (AisleriotBoard *board,
-                 int cursor_type,
-                 ArClutterEmbed *embed)
-{
-  ar_clutter_embed_set_cursor (embed, (ArCursorType) cursor_type);
-}
-
-static void
-board_error_bell_cb (AisleriotBoard *board,
-                     ArClutterEmbed *embed)
-{
-#if GTK_CHECK_VERSION (2, 12, 0) || (defined (HAVE_HILDON) && !defined(HAVE_MAEMO_3))
-  gtk_widget_error_bell (GTK_WIDGET (embed));
-#endif
-}
-
-static void
-embed_size_allocate_cb (ArClutterEmbed *embed,
-                        GtkAllocation *allocation,
-                        AisleriotWindow *window)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-
-  clutter_actor_set_size (priv->board_actor, allocation->width, allocation->height);
-}
-
-#endif /* HAVE_CLUTTER */
-
 /* Class implementation */
 
 #ifdef HAVE_HILDON
@@ -2246,17 +2131,15 @@ aisleriot_window_init (AisleriotWindow *window)
     { "About", GTK_STOCK_ABOUT, NULL, NULL,
       ACTION_TOOLTIP (N_("About this game")),
       G_CALLBACK (help_about_cb) },
-#ifdef ENABLE_CARD_THEMES_INSTALLER
+#ifndef HAVE_HILDON
     { "InstallThemes", NULL, N_("Install card themes…"), NULL,
       ACTION_TOOLTIP (N_("Install new card themes from the distribution packages repositories")),
       G_CALLBACK (install_themes_cb) },
-#endif /* ENABLE_CARD_THEMES_INSTALLER */
+#endif /* HAVE_HILDON */
 
     /* Toolbar-only actions */
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
     { "LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL,
       G_CALLBACK (leave_fullscreen_cb) },
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 #ifndef HAVE_HILDON
     { "ThemeMenu", NULL, N_("_Card Style"), NULL, NULL, NULL },
 #endif /* !HAVE_HILDON */
@@ -2321,12 +2204,6 @@ aisleriot_window_init (AisleriotWindow *window)
       G_CALLBACK (sound_toggle_cb),
       FALSE /* not active by default */ },
 #endif /* ENABLE_SOUND */
-#ifdef HAVE_CLUTTER
-   { "Animations", NULL, N_("_Animations"), NULL,
-      ACTION_TOOLTIP (N_("Whether or not to animate card moves")),
-      G_CALLBACK (animations_toggle_cb),
-      FALSE /* not active by default */ },
-#endif /* HAVE_CLUTTER */
   };
 
   static const char names[][16] = {
@@ -2338,12 +2215,10 @@ aisleriot_window_init (AisleriotWindow *window)
     "OptionsMenu",
     "Deal",
     "Hint",
+    "LeaveFullscreen",
 #ifdef HAVE_HILDON
     "AccelUndoMove",
 #endif /* HAVE_HILDON */
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
-    "LeaveFullscreen",
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
   };
 
   static const char ui_description[] =
@@ -2409,7 +2284,7 @@ aisleriot_window_init (AisleriotWindow *window)
           "<separator/>"
           "<menu action='ThemeMenu'>"
             "<placeholder name='ThemesPH'/>"
-#ifdef ENABLE_CARD_THEMES_INSTALLER
+#ifndef HAVE_HILDON
             "<separator/>"
             "<menuitem action='InstallThemes'/>"
 #endif
@@ -2424,9 +2299,6 @@ aisleriot_window_init (AisleriotWindow *window)
           "<menuitem action='ClickToMove'/>"
 #ifdef ENABLE_SOUND
           "<menuitem action='Sound'/>"
-#endif
-#ifdef HAVE_CLUTTER
-          "<menuitem action='Animations'/>"
 #endif
         "</menu>"
         "<menu action='OptionsMenu'/>"
@@ -2463,10 +2335,8 @@ aisleriot_window_init (AisleriotWindow *window)
         "<toolitem action='RedoMove'/>"
         "<toolitem action='Deal'/>"
         "<toolitem action='Hint'/>"
-#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
         "<separator name='LeaveFullscreenSep' expand='true'/>"
         "<toolitem action='LeaveFullscreen'/>"
-#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 #ifdef ENABLE_DEBUG_UI
         "<toolitem action='DebugGameFirst'/>"
         "<toolitem action='DebugGamePrev'/>"
@@ -2493,9 +2363,6 @@ aisleriot_window_init (AisleriotWindow *window)
   GtkStatusbar *statusbar;
   GtkWidget *statusbar_hbox, *label, *time_box;
 #endif
-#ifdef HAVE_CLUTTER
-  ClutterContainer *stage;
-#endif
 
   g_assert (G_N_ELEMENTS (names) == LAST_ACTION);
 
@@ -2509,34 +2376,7 @@ aisleriot_window_init (AisleriotWindow *window)
 
   priv->board_style = ar_style_new ();
 
-#ifdef HAVE_CLUTTER
-  priv->board = ar_clutter_embed_new (priv->board_style);
-
-  priv->baize_actor = aisleriot_baize_new ();
-
-  stage = CLUTTER_CONTAINER (gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (priv->board)));
-  clutter_container_add (stage, priv->baize_actor, NULL);
-  /* FIXMEchpe: how to ensure this is ALWAYS the lowest actor? */
-  clutter_actor_lower_bottom (priv->baize_actor);
-
-  priv->board_actor = aisleriot_board_new (priv->board_style, priv->game);
-  clutter_container_add (stage, priv->board_actor, NULL);
-
-  /* FIXMEchpe */
-  clutter_stage_set_key_focus (CLUTTER_STAGE (stage), priv->board_actor);
-
-  g_signal_connect_after (priv->board, "size-allocate",
-                          G_CALLBACK (embed_size_allocate_cb), window);
-
-  g_signal_connect (priv->board_actor, "request-cursor",
-                    G_CALLBACK (board_cursor_cb), priv->board);
-  g_signal_connect (priv->board_actor, "error-bell",
-                    G_CALLBACK (board_error_bell_cb), priv->board);
-
-  /* FIXMEchpe: unref baize & board_actor here? */
-#else
   priv->board = AISLERIOT_BOARD (aisleriot_board_new (priv->board_style, priv->game));
-#endif /* HAVE_CLUTTER */
 
   theme_name = games_conf_get_string (NULL, aisleriot_conf_get_key (CONF_THEME), NULL);
   theme = ar_card_themes_get_theme_by_name (priv->theme_manager, theme_name);
@@ -2588,13 +2428,8 @@ aisleriot_window_init (AisleriotWindow *window)
 
   priv->game_message_id = gtk_statusbar_get_context_id (priv->statusbar, "board-message");
 
-#ifdef HAVE_CLUTTER
-  g_signal_connect (priv->board_actor, "status-message",
-                    G_CALLBACK (board_status_message_cb), window);
-#else
   g_signal_connect (priv->board, "status-message",
                     G_CALLBACK (board_status_message_cb), window);
-#endif
 
 #if GTK_CHECK_VERSION (2, 11, 0)
   gtk_statusbar_set_has_resize_grip (priv->statusbar, TRUE);
@@ -2662,27 +2497,6 @@ aisleriot_window_init (AisleriotWindow *window)
   priv->main_menu = gtk_ui_manager_get_widget (priv->ui_manager, MAIN_MENU_UI_PATH);
   priv->toolbar = gtk_ui_manager_get_widget (priv->ui_manager, TOOLBAR_UI_PATH);
 
-#ifdef MAEMO_TOOLBAR_BANNER
-{
-  GtkToolItem *tool_item;
-
-  /* It's a bit ugly to mess with the toolbar like this,
-   * but it's simpler than to implement a dedicated action
-   * which creates a tool item which ellipsises its label...
-   */
-  tool_item = gtk_tool_item_new ();
-  gtk_tool_item_set_expand (tool_item, TRUE);
-
-  priv->game_message_label = GTK_LABEL (gtk_label_new (NULL));
-  gtk_misc_set_alignment (GTK_MISC (priv->game_message_label), 1.0, 0.5);
-  gtk_label_set_ellipsize (priv->game_message_label, PANGO_ELLIPSIZE_END);
-  gtk_container_add (GTK_CONTAINER (tool_item), GTK_WIDGET (priv->game_message_label));
-
-  gtk_toolbar_insert (GTK_TOOLBAR (priv->toolbar), tool_item, -1);
-  gtk_widget_show_all (GTK_WIDGET (tool_item));
-}
-#endif /* MAEMO_TOOLBAR_BANNER */
-
   /* Defer building the card themes menu until its parent's menu is opened */
 #ifdef HAVE_HILDON
   /* FIXMEchpe check this works! */
@@ -2746,26 +2560,19 @@ aisleriot_window_init (AisleriotWindow *window)
                                 priv->statusbar_visible);
 #endif /* !HAVE_HILDON */
 
-#ifdef ENABLE_CARD_THEMES_INSTALLER
+#ifndef HAVE_HILDON
   action = gtk_action_group_get_action (priv->action_group, "InstallThemes");
-  gtk_action_set_sensitive (action, ar_card_themes_can_install_themes (priv->theme_manager));
-#endif /* ENABLE_CARD_THEMES_INSTALLER */
+  gtk_action_set_visible (action, ar_card_themes_can_install_themes (priv->theme_manager));
+#endif /* HAVE_HILDON */
 
   set_fullscreen_actions (window, FALSE);
 
-#ifdef HAVE_CLUTTER
-  action = gtk_action_group_get_action (priv->action_group, "Animations");
-  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-                                games_conf_get_boolean (NULL, aisleriot_conf_get_key (CONF_ANIMATIONS), NULL));
-
-#endif /* HAVE_CLUTTER */
-
-#if defined(HAVE_CLUTTER) || (defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0))
+#if defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0)
   /* Set the action visibility and listen for animation and sound mode changes */
   screen_changed_cb (GTK_WIDGET (window), NULL, window);
   g_signal_connect (window, "screen-changed",
                     G_CALLBACK (screen_changed_cb), window);
-#endif /* HAVE_CLUTTER || ENABLE_SOUND && GTK+ >= 2.14.0 */
+#endif /* ENABLE_SOUND && GTK+ >= 2.14.0 */
 
   /* Now set up the widgets */
   main_vbox = gtk_vbox_new (FALSE, 0);
@@ -2847,12 +2654,6 @@ aisleriot_window_dispose (GObject *object)
   AisleriotWindow *window = AISLERIOT_WINDOW (object);
   AisleriotWindowPrivate *priv = window->priv;
   
-#ifdef HAVE_CLUTTER
-  g_signal_handlers_disconnect_by_func (gtk_widget_get_settings (GTK_WIDGET (window)),
-                                        G_CALLBACK (settings_changed_cb),
-                                        window);
-#endif /* HAVE_CLUTTER */
-
 #ifndef HAVE_HILDON
   if (priv->hint_dialog) {
     gtk_widget_destroy (priv->hint_dialog);
@@ -2893,10 +2694,6 @@ aisleriot_window_finalize (GObject *object)
   AisleriotWindow *window = AISLERIOT_WINDOW (object);
   AisleriotWindowPrivate *priv = window->priv;
 
-#ifdef HAVE_CLUTTER
-  g_object_unref (priv->board_style);
-#endif /* HAVE_CLUTTER */
-
   if (priv->theme) {
     g_object_unref (priv->theme);
   }
