@@ -1,7 +1,8 @@
-/* 
+/*
  *   Gnome Nibbles: Gnome Worm Game
- *   Written by Sean MacIsaac <sjm@acm.org>, Ian Peters <itp@gnu.org>
- * 
+ *   Written by Sean MacIsaac <sjm@acm.org>, Ian Peters <itp@gnu.org>,
+ *              Guillaume Beland <guillaume.beland@gmail.com>
+ *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -21,7 +22,9 @@
 #define _BONUS_H_
 
 #include <config.h>
-#include <gnome.h>
+
+#include <gtk/gtk.h>
+#include <clutter/clutter.h>
 
 #define BONUSNONE	0
 #define BONUSREGULAR	1
@@ -33,17 +36,18 @@
 #define BONUSSWITCH	7
 
 typedef struct {
-	gint x, y;
-	guint type;
-	gint fake;
-	gint countdown;
+  gint x, y;
+  guint type;
+  gint fake;
+  gint countdown;
+  ClutterActor *actor;
 } GnibblesBonus;
 
 GnibblesBonus *gnibbles_bonus_new (gint t_x, gint t_y, gint t_type,
-		gint t_fake, gint t_countdown);
+				   gint t_fake, gint t_countdown);
 
-void gnibbles_bonus_draw (GnibblesBonus *bonus);
+void gnibbles_bonus_draw (GnibblesBonus * bonus);
 
-void gnibbles_bonus_erase (GnibblesBonus *bonus);
+void gnibbles_bonus_erase (GnibblesBonus * bonus);
 
 #endif
